#include "storage.h"

bool Storage::isMounted = false;
uint8_t Storage::currentBaseSpeed = DEFAULT_BASE_SPEED;
uint8_t Storage::currentTurnSpeed = DEFAULT_TURN_SPEED;
uint8_t Storage::currentRotateSensitive = DEFAULT_ROTATE_SENSITIVE;
unsigned int Storage::currentSensitivity = DEFAULT_SENSITIVITY;
float Storage::currentKp = DEFAULT_KP;
float Storage::currentKi = DEFAULT_KI;
float Storage::currentKd = DEFAULT_KD;
String Storage::currentName = DEVICE_NAME;

void Storage::init()
{
#if STORAGE_ENABLE
  if (!SPIFFS.begin(true))
  {
    Serial.println("SPIFFS Mount Failed");
    isMounted = false;
  }
  else
  {
    Serial.println("SPIFFS Mount Success");
    isMounted = true;
  }
#else
  Serial.println("Storage disabled");
  isMounted = false;
#endif
}

void Storage::setTrackingParams(uint8_t baseSpeed, uint8_t turnSpeed, uint8_t rotateSensitive)
{
  currentBaseSpeed = baseSpeed;
  currentTurnSpeed = turnSpeed;
  currentRotateSensitive = rotateSensitive;

#if STORAGE_ENABLE
  if (!isMounted)
    return;

  File file = SPIFFS.open("/tracking.txt", "w");
  file.println(baseSpeed);
  file.println(turnSpeed);
  file.println(rotateSensitive);
  file.close();
#endif
}

void Storage::getTrackingParams(uint8_t &baseSpeed, uint8_t &turnSpeed, uint8_t &rotateSensitive)
{
#if STORAGE_ENABLE
  if (!isMounted || !SPIFFS.exists("/tracking.txt"))
  {
    baseSpeed = currentBaseSpeed;
    turnSpeed = currentTurnSpeed;
    rotateSensitive = currentRotateSensitive;
    return;
  }

  File file = SPIFFS.open("/tracking.txt", "r");
  baseSpeed = file.readStringUntil('\n').toInt();
  turnSpeed = file.readStringUntil('\n').toInt();
  rotateSensitive = file.readStringUntil('\n').toInt();
  file.close();

  // 更新当前值
  currentBaseSpeed = baseSpeed;
  currentTurnSpeed = turnSpeed;
  currentRotateSensitive = rotateSensitive;
#else
  baseSpeed = currentBaseSpeed;
  turnSpeed = currentTurnSpeed;
  rotateSensitive = currentRotateSensitive;
#endif
}

void Storage::setPID(float kp, float ki, float kd)
{
  currentKp = kp;
  currentKi = ki;
  currentKd = kd;

#if STORAGE_ENABLE
  if (!isMounted)
    return;

  File file = SPIFFS.open("/pid.txt", "w");
  file.println(kp);
  file.println(ki);
  file.println(kd);
  file.close();
#endif
}

void Storage::getPID(float &kp, float &ki, float &kd)
{
#if STORAGE_ENABLE
  if (!isMounted || !SPIFFS.exists("/pid.txt"))
  {
    kp = currentKp;
    ki = currentKi;
    kd = currentKd;
    return;
  }

  File file = SPIFFS.open("/pid.txt", "r");
  kp = file.readStringUntil('\n').toFloat();
  ki = file.readStringUntil('\n').toFloat();
  kd = file.readStringUntil('\n').toFloat();
  file.close();

  currentKp = kp;
  currentKi = ki;
  currentKd = kd;
#else
  kp = currentKp;
  ki = currentKi;
  kd = currentKd;
#endif
}

unsigned int Storage::getSensitivity()
{
#if STORAGE_ENABLE
  if (!isMounted)
    return 0xFF;

  File file = SPIFFS.open("/sensitivity.txt", "r");
  if (!file)
    return 0xFF;

  String sensitivity = file.readStringUntil('\n');
  file.close();

  return sensitivity.toInt();
#else
  return 0xFF;
#endif
}

void Storage::setSensitivity(unsigned int sensitivity)
{
#if STORAGE_ENABLE
  if (!isMounted)
    return;

  File file = SPIFFS.open("/sensitivity.txt", "w");
  file.println(sensitivity);
  file.close();
#endif
}

void Storage::setName(String name)
{
#if STORAGE_ENABLE
  if (!isMounted)
    return;

  File file = SPIFFS.open("/name.txt", "w");
  file.println(name);
  file.close();
#endif
}

String Storage::getName()
{
#if STORAGE_ENABLE
  if (!isMounted)
    return DEVICE_NAME;

  File file = SPIFFS.open("/name.txt", "r");
  String name = file.readStringUntil('\n');
  file.close();

  return name;
#else
  return DEVICE_NAME;
#endif
}
